﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace wscf.lightblue.Build
{
	public delegate void LogHandler(object sender, string message);
	public delegate void ErrorHandler(object sender, string message, Exception exc);
	
	public class Activity
	{
		public string Name { get; set; }

		protected bool m_IsError = false;
		
		public bool IsError
		{
			get
			{
				return m_IsError;
			}
		}

		public static event LogHandler Log;

		protected static int m_LogIndents = 0;
		public static void LogIndent() 
		{ 
			m_LogIndents++; 
		}

		public static void LogUnIndent()
		{
			if (m_LogIndents > 0)
			{
				m_LogIndents--;
			}
		}

		public static string IndentString
		{
			get
			{
				string s = "";
				for (int i = 0; i < m_LogIndents; i++)
				{
					s += "\t";
				}
				return s;
			}
		}

		protected virtual void OnLog(string s)
		{
			if (Log != null)
			{
				StringBuilder sb = new StringBuilder();
				if (s.Contains("\n"))
				{
					foreach (string line in s.Split('\n'))
					{
						sb.AppendLine(Activity.IndentString + line.Replace("\r", ""));
					}
					s = sb.ToString();
				}
				else
				{
					s = Activity.IndentString + s;
				}
				Log(this, s);
			}
		}

		public static event ErrorHandler Error;

		protected virtual void OnError(string message)
		{
			OnError(message, null);
		}

		protected virtual void OnError(Exception e)
		{
			OnError(null, e);
		}

		protected virtual void OnError(string message, Exception e)
		{
			m_IsError = true;
			if (Error != null)
			{
				if (string.IsNullOrEmpty(message) && e != null)
				{
					message = "Error: " + e.Message;
				}
				Error(this, Activity.IndentString + message, e);
			}
			else
			{
				throw e;
			}
		}

		public virtual void Execute()
		{
			m_IsError = false;
		}

		public override string ToString()
		{
			return Name;
		}

		public static string GetBuildTempFolder()
		{
			string s = Path.Combine(
				Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData),
				@"wscf.lightblue\build");
			if (!Directory.Exists(s))
			{
				Directory.CreateDirectory(s);
			}
			return s;
		}

		protected static string GetProgramFilesFolder()
		{
			if (Environment.Is64BitOperatingSystem)
			{
				return Environment.GetFolderPath(Environment.SpecialFolder.ProgramFilesX86);
			}
			return Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles);
		}

	}
}
