﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace wscf.lightblue.Build
{
	public class GenerateServiceContractCode : Activity
	{
		public List<string> ReferencedAssemblies { get; set; }

		public List<string> ServiceContracts { get; set; }

		public string ServiceAssembliesFolder { get; set; }

        [DefaultValue(false)]
        public bool ContinueOnError { get; set; }

		public override void Execute()
		{
			OnLog(ToString());
			base.Execute();
			if (ServiceContracts != null)
			{
				foreach (string file in ServiceContracts)
				{
					if (Path.GetExtension(file).ToLower() == ".wsdl")
					{
						InvokeProcess p = new InvokeProcess
						{
							Name = "Generate Code from Service Contract",
							FileName = Path.Combine(GetProgramFilesFolder(), @"Capgemini\wscf.lightblue\wscf.exe"),
							Arguments = string.Format("/m:\"{0}\" /out:\"{1}\" /n:*,{2} {3}",
								file,
								Path.Combine(GetBuildTempFolder(), Path.ChangeExtension(Path.GetFileName(file), ".cs")),
								CodeGenerator.XmlNamespace2ClsNamespace(CodeGenerator.GetWsdlTargetNamespace(file)),
								GetReferences()),
							WorkingDirectory = ServiceAssembliesFolder
						};
						OnLog(p.FileName + " " + p.Arguments);
						p.Execute();
						if (p.IsError || p.ConsoleOutput.ToLower().Contains("error:"))
						{
							OnError(p.ConsoleOutput);
							m_IsError = true;
							if (!ContinueOnError)
							{
								break;
							}
						}
					}
				}
			}
		}

		private string GetReferences()
		{
			string r = string.Empty;
			if (ReferencedAssemblies != null)
			{
				foreach (string file in ReferencedAssemblies)
				{
					if(!string.IsNullOrEmpty(r))
					{
						r += "\",\"";
					}
					r += Path.Combine(ServiceAssembliesFolder, Path.GetFileName(file));
				}
			}
			if (!string.IsNullOrEmpty(r))
			{
				r = "/r:\"" + r + "\"";
			}
			return r;
		}
	}
}
