﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Collections.Specialized;

namespace wscf.lightblue.Build
{
	public class InvokeProcess : Activity
	{
		private StringBuilder m_ConsoleOutput = new StringBuilder();

		public string ConsoleOutput
		{
			get
			{
				return m_ConsoleOutput.ToString();
			}
		}

		public string FileName { get; set; }

		public string Arguments { get; set; }

		public string WorkingDirectory { get; set; }

		public StringDictionary EnvironmentVariables { get; set; }

		private bool m_UseShellExecute = false;

		public bool UseShellExecute 
		{ 
			get 
			{ 
				return m_UseShellExecute;
			}
			set 
			{ 
				m_UseShellExecute = value; 
			} 
		}

		public int ExitCode { get; set; }

		public override void Execute()
		{
			base.Execute();
			try
			{
				Process p = new Process();
				p.StartInfo.FileName = FileName;
				p.StartInfo.Arguments = Arguments;
				p.StartInfo.WorkingDirectory = WorkingDirectory;
				p.StartInfo.RedirectStandardOutput = !UseShellExecute;
				p.StartInfo.RedirectStandardError = !UseShellExecute;
				if (!UseShellExecute && EnvironmentVariables != null)
				{
					foreach (string key in EnvironmentVariables.Keys)
					{
						if (!p.StartInfo.EnvironmentVariables.ContainsKey(key))
						{
							p.StartInfo.EnvironmentVariables.Add(key, EnvironmentVariables[key]);
						}
						else
						{
							p.StartInfo.EnvironmentVariables[key] = EnvironmentVariables[key];
						}
					}
				}
				p.EnableRaisingEvents = true;
				p.StartInfo.CreateNoWindow = true;
				p.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
				p.StartInfo.UseShellExecute = UseShellExecute;
				if (p.StartInfo.RedirectStandardOutput)
				{
					p.OutputDataReceived += proc_DataReceived;
				}
				if (p.StartInfo.RedirectStandardError)
				{
					p.ErrorDataReceived += proc_DataReceived;
				}
				m_ConsoleOutput.Clear();
				p.Start();
				if (p.StartInfo.RedirectStandardOutput)
				{
					p.BeginOutputReadLine();
				}
				if (p.StartInfo.RedirectStandardError)
				{
					p.BeginErrorReadLine();
				}
				p.WaitForExit();
				p.Refresh();
				if(!string.IsNullOrEmpty(m_ConsoleOutput.ToString()))
				{
					OnLog(m_ConsoleOutput.ToString());
				}
				ExitCode = p.ExitCode;
			}
			catch (Exception e)
			{
				OnError(e);
			}
		}

		private void proc_DataReceived(object sender, DataReceivedEventArgs e)
		{
			if (e.Data != null)
			{
				lock (m_ConsoleOutput)
				{
					m_ConsoleOutput.AppendLine(e.Data);
				}
			}
		}

		public override string ToString()
		{
			return string.Format("{0} [{1} {2}]", 
				base.ToString(), this.FileName, this.Arguments);
		}
	}
}
