﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Collections.Specialized;
using Microsoft.Win32;
using System.Diagnostics;

namespace wscf.lightblue.Build
{
	public class InvokeVisualStudioTool : Activity
	{
		public string FileName { get; set; }
		public string Arguments { get; set; }

		public override void Execute()
		{
			base.Execute();
			InvokeShellCommand c= new InvokeShellCommand
			{
				Name = Name,
				Command = string.Format("call \"{0}\"\n{1} {2}", 
				Path.Combine(GetVisualStudioInstallationPath(), "..\\..\\VC\\vcvarsall.bat"), FileName, Arguments)
			};
			OnLog(c.Command);
			c.Execute();
			if (c.IsError)
			{
				m_IsError = true;
			}
		}

		private string GetVisualStudioInstallationPath()
		{
			string s = string.Empty;
			s = GetVisualStudioInstallationPath("10.0");
			if (string.IsNullOrEmpty(s))
			{
				s = GetVisualStudioInstallationPath("9.0");
			}
			if (string.IsNullOrEmpty(s))
			{
				s = GetVisualStudioInstallationPath("11.0");
			}
			if (string.IsNullOrEmpty(s))
			{
				throw new Exception("Visual Studio must be installed");
			}
			return s;
		}

		private string GetVisualStudioInstallationPath(string version)
		{
			string installationPath = null;
			if (Environment.Is64BitOperatingSystem)
			{
				installationPath = (string)Registry.GetValue(
					string.Format(@"HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Microsoft\VisualStudio\{0}\", version),
					"InstallDir",
					null);
			}
			else
			{
				installationPath = (string)Registry.GetValue(
					string.Format(@"HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\VisualStudio\{0}\", version),
					"InstallDir",
					null);
			}
			return installationPath;
		}

		public override string ToString()
		{
			return string.Format("{0} [{1} {2}]", base.ToString(), FileName, Arguments);
		}

	}
}
