﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace wscf.lightblue.Build
{
	public class XCopy : Activity
	{
		[DefaultValue("*.*")]
		public string FilesPattern { get; set; }
		[DefaultValue(false)]
		public bool RecurseFolders { get; set; }
		[DefaultValue(false)]
		public bool CopyEmptyFolders { get; set; }
		[DefaultValue(false)]
		public bool CopyHiddenAndSystemFiles { get; set; }
		[DefaultValue(false)]
		public bool OverwriteExistingFiles { get; set; }
		[DefaultValue(false)]
		public bool CopyOnlyIfSourceIsNewer { get; set; }
		[DefaultValue("*")]
		public string FoldersPattern { get; set; }
		[DefaultValue("*")]
		public string SourceFolder { get; set; }
		[DefaultValue("*")]
		public string DestinationFolder { get; set; }

		public override void Execute()
		{
			base.Execute();
			OnLog(ToString());
			try
			{
				LogIndent();
				var options = new ZetaFolderXCopyOptions
				{
					FilesPattern = this.FilesPattern,
					RecurseFolders = this.RecurseFolders,
					CopyEmptyFolders = true,
					CopyHiddenAndSystemFiles = true,
					OverwriteExistingFiles = true,
					CopyOnlyIfSourceIsNewer = false,
					FoldersPattern = "*"
				}.AddExcludeSubStrings(
					"\\.svn\\",
					"\\_svn\\",
					"\\_Temporary\\");

				var xc = new ZetaFolderXCopy();
				xc.Copy(
					SourceFolder,
					DestinationFolder,
					options);
			}
			catch (Exception exc)
			{
				OnError(exc);
			}
			finally
			{
				LogUnIndent();
			}
		}

		public override string ToString()
		{
			return string.Format(@"{0} [xcopy {1}\{2} {3}]",
				base.ToString(), SourceFolder, FilesPattern, DestinationFolder);
		}
	}
}
